if ( !("Spells" in getroottable()) ) {
    ::Spells <- { version = "1.0.0" };
}

::Spells.CustomSpells <- {};
::Spells.IsDev <- function() { return developer() >= 1; }

// ------------------------------
// SPELL IDs
// these determine what spell is used, or what icon gets shown for custom spells
// ------------------------------

::Spells.ESpellID <- {
    SPELL_FIREBALL          = 0,
	SPELL_BATS              = 1,
	SPELL_SELF_HEAL         = 2,
	SPELL_MIRV              = 3,
	SPELL_BLAST_JUMP        = 4,
	SPELL_STEALTH           = 5,
	SPELL_TELEPORT          = 6,
	SPELL_LIGHTNING_BALL    = 7,
	SPELL_MOVEMENT_BUFF     = 8,
	SPELL_METEOR_SWARM      = 9,
	SPELL_MONOCULUS         = 10,
	SPELL_SKELETON_HORDE    = 11,
	SPELL_KART_BOXING_GLOVE = 12,
	SPELL_KART_PARACHUTE    = 13,
	SPELL_KART_SELF_HEAL    = 14,
	SPELL_KART_BOMB         = 15
}

// ------------------------------
// BASE SPELLS
// for quick definition and reuse
// ------------------------------

class Spells.SpellBase {
    spell_id = Spells.ESpellID.SPELL_FIREBALL;
	is_custom_spell = false;
	is_rare_spell = false;
}

::Spells.ESpells <- {};
class Spells.ESpells.SPELL_FIREBALL 	        extends Spells.SpellBase { spell_id = Spells.ESpellID.SPELL_FIREBALL; }
class Spells.ESpells.SPELL_BATS 		        extends Spells.SpellBase { spell_id = Spells.ESpellID.SPELL_BATS; }
class Spells.ESpells.SPELL_SELF_HEAL 	        extends Spells.SpellBase { spell_id = Spells.ESpellID.SPELL_SELF_HEAL; }
class Spells.ESpells.SPELL_MIRV 		        extends Spells.SpellBase { spell_id = Spells.ESpellID.SPELL_MIRV; }
class Spells.ESpells.SPELL_BLAST_JUMP 	        extends Spells.SpellBase { spell_id = Spells.ESpellID.SPELL_BLAST_JUMP; }
class Spells.ESpells.SPELL_STEALTH 		        extends Spells.SpellBase { spell_id = Spells.ESpellID.SPELL_STEALTH; }
class Spells.ESpells.SPELL_TELEPORT 	        extends Spells.SpellBase { spell_id = Spells.ESpellID.SPELL_TELEPORT; }
class Spells.ESpells.SPELL_KART_BOXING_GLOVE    extends Spells.SpellBase { spell_id = Spells.ESpellID.SPELL_KART_BOXING_GLOVE; }
class Spells.ESpells.SPELL_KART_PARACHUTE       extends Spells.SpellBase { spell_id = Spells.ESpellID.SPELL_KART_PARACHUTE; }
class Spells.ESpells.SPELL_KART_SELF_HEAL       extends Spells.SpellBase { spell_id = Spells.ESpellID.SPELL_KART_SELF_HEAL; }
class Spells.ESpells.SPELL_KART_BOMB            extends Spells.SpellBase { spell_id = Spells.ESpellID.SPELL_KART_BOMB; }

// rare spells get marked specifically so they get the correct mercenary response voice lines

class Spells.ESpells.SPELL_LIGHTNING_BALL extends Spells.SpellBase {
	spell_id = Spells.ESpellID.SPELL_LIGHTNING_BALL;
	is_rare_spell = true;
}

class Spells.ESpells.SPELL_MOVEMENT_BUFF extends Spells.SpellBase {
	spell_id = Spells.ESpellID.SPELL_MOVEMENT_BUFF;
	is_rare_spell = true;
}

class Spells.ESpells.SPELL_METEOR_SWARM extends Spells.SpellBase {
	spell_id = Spells.ESpellID.SPELL_METEOR_SWARM;
	is_rare_spell = true;
}

class Spells.ESpells.SPELL_MONOCULUS extends Spells.SpellBase {
	spell_id = Spells.ESpellID.SPELL_MONOCULUS;
	is_rare_spell = true;
}

class Spells.ESpells.SPELL_SKELETON_HORDE extends Spells.SpellBase {
	spell_id = Spells.ESpellID.SPELL_SKELETON_HORDE;
	is_rare_spell = true;
}

// ------------------------------
// BASE SPELL POOLS
// ------------------------------

::Spells.DefaultCommonSpellList <- [
	{ "spell": Spells.ESpells.SPELL_FIREBALL, 		"weight": 1, "charges": 2},
	{ "spell": Spells.ESpells.SPELL_BATS, 			"weight": 1, "charges": 2},
	{ "spell": Spells.ESpells.SPELL_SELF_HEAL, 		"weight": 1, "charges": 1},
	{ "spell": Spells.ESpells.SPELL_MIRV, 			"weight": 1, "charges": 1},
	{ "spell": Spells.ESpells.SPELL_BLAST_JUMP, 	"weight": 1, "charges": 2},
	{ "spell": Spells.ESpells.SPELL_STEALTH, 		"weight": 1, "charges": 1},
	{ "spell": Spells.ESpells.SPELL_TELEPORT, 		"weight": 1, "charges": 2},
];

::Spells.DefaultKartSpellList <- [
	{ "spell": Spells.ESpells.SPELL_FIREBALL, 		    "weight": 1, "charges": 1},
	{ "spell": Spells.ESpells.SPELL_KART_PARACHUTE,     "weight": 1, "charges": 1},
	{ "spell": Spells.ESpells.SPELL_KART_BOXING_GLOVE,  "weight": 1, "charges": 1},
	{ "spell": Spells.ESpells.SPELL_KART_SELF_HEAL,     "weight": 1, "charges": 1},
	{ "spell": Spells.ESpells.SPELL_KART_BOMB, 		    "weight": 1, "charges": 1}
];

::Spells.DefaultRareSpellList <- [
	{ "spell": Spells.ESpells.SPELL_LIGHTNING_BALL, "weight": 1, "charges": 1},
	{ "spell": Spells.ESpells.SPELL_MOVEMENT_BUFF, 	"weight": 1, "charges": 1},
	{ "spell": Spells.ESpells.SPELL_METEOR_SWARM, 	"weight": 1, "charges": 1},
	{ "spell": Spells.ESpells.SPELL_MONOCULUS, 		"weight": 1, "charges": 1},
	{ "spell": Spells.ESpells.SPELL_SKELETON_HORDE, "weight": 1, "charges": 1}
];