function Precache() {
    PrecacheScriptSound("Halloween.spell_fireball_cast");
}

function GetPlayerSpellbook(player) {
    local weapons_count = NetProps.GetPropArraySize(player, "m_hMyWeapons")
    for (local i = 0; i < weapons_count; i++) {
        local weapon = NetProps.GetPropEntityArray(player, "m_hMyWeapons", i)
        if(weapon != null && weapon.IsValid() && weapon.GetClassname() == "tf_weapon_spellbook") {
            return weapon;
        }
    }
    return null;
}

function HandleCustomSpells(player) {
    local lifestate = NetProps.GetPropInt(player, "m_lifeState");
    if(lifestate != 0) return;

    local spellbook = GetPlayerSpellbook(player);
    if(spellbook == null) return;

    local index     = NetProps.GetPropInt(spellbook, "m_iSelectedSpellIndex");
    local charges   = NetProps.GetPropInt(spellbook, "m_iSpellCharges");

    if((index != 20 && index != -20) || charges <= 0) return;
    StopSoundOn( "Player.DenyWeaponSelection", player);

    local nextTime = NetProps.GetPropFloat(spellbook, "m_flTimeNextSpell");

    player.ValidateScriptScope();
    local scope = player.GetScriptScope();

    if(!scope.spells.can_cast_spell) return;

    local spell = scope.spells.spellbook_spell;

    if(player.IsUsingActionSlot()) {
        // prevent players spamming or holding down the action slot key to cast multiple spells
        if(nextTime > Time() || scope.spells.cast_spell_this_action) return;

        scope.spells.cast_spell_this_action = true;

        // we're casting a spell even though we don't have one, so fake the spellbook's attack
        player.Weapon_Switch(spellbook);
        spellbook.PrimaryAttack();
        StopSoundOn( "Player.DenyWeaponSelection", player);

        NetProps.SetPropFloat(spellbook, "m_flTimeNextSpell", Time() + 0.5);

        // spells don't update charges unless the ID is in bounds
        NetProps.SetPropInt(spellbook, "m_iSelectedSpellIndex", spell.spell_id);

        EntFireByHandle(player, "SpeakResponseConcept", spell.cast_response, 0, null, null);

        NetProps.SetPropInt(spellbook, "m_iSpellCharges", charges-1);

        EntFireByHandle(self, "CallScriptFunction", "ResetPlayerSpell", 0.15, player, player);
        EntFireByHandle(self, "CallScriptFunction", "CastCustomSpell", spell.cast_delay, player, player);

        if(charges-1 == 0) NetProps.SetPropInt(spellbook, "m_iSelectedSpellIndex", -1);

    } else {
        scope.spells.cast_spell_this_action = false;
    }
}

function CastCustomSpell() {
    local player = activator;
    local scope = player.GetScriptScope();
    local spell = scope.spells.spellbook_spell;

    scope.spells.active_spell_effects.push(spell);

    spell.is_active = true;
    spell.spell_start_time = Time();

    PrecacheScriptSound(spell.cast_sound);
    player.EmitSound(spell.cast_sound);
    spell.OnCast(player);
}


function ResetPlayerSpell() {
    local player = activator;
    player.ValidateScriptScope();
    local scope = player.GetScriptScope();
    local spellbook = GetPlayerSpellbook(player);
    local spell = scope.spells.spellbook_spell;

    // play out the rest of the firing animation
    local viewmodel = NetProps.GetPropEntity(player, "m_hViewModel");
    local seq = viewmodel.LookupSequence("spell_fire");
    viewmodel.ResetSequence(seq);

    // set the spell index back out of bounds to prevent regular casting
    NetProps.SetPropInt(spellbook, "m_iSelectedSpellIndex", 20);

    if(spellbook == null) return;
}

::MaxPlayers <- MaxClients().tointeger();

function CustomCastingInit() {
    for (local i = 1; i <= MaxPlayers; i++)
    {
        local player = PlayerInstanceFromIndex(i);
        if (player == null) continue;

        player.ValidateScriptScope();
        CastingInitPlayer(player);

    }
}

function CastingInitPlayer(player) {
    local scope = player.GetScriptScope();
    scope.spells <- {
        spellbook_spell = null,
        active_spell_effects = [],
        can_pickup_spell = true,
        can_cast_spell = true,
        cast_spell_this_action = false,
    };
}

function CastingThink() {
    for (local i = 1; i <= MaxPlayers; i++)
    {
        local player = PlayerInstanceFromIndex(i);
        if (player == null) return;
        HandleCustomSpells(player);
    }
}

function CustomSpellsThink() {
    for (local i = 1; i <= MaxPlayers; i++)
    {
        local player = PlayerInstanceFromIndex(i);
        if (player == null) continue;

        player.ValidateScriptScope();
        local scope = player.GetScriptScope();

        if(!scope.rawin("spells")) CastingInitPlayer(player);

        foreach(spell in scope.spells.active_spell_effects) {
            if(!spell || !spell.is_custom_spell) continue;

            local lifestate = NetProps.GetPropInt(player, "m_lifeState");
            if(lifestate != 0) {
                if(spell.is_active) spell.OnEnd(player);
                continue;
            }

            if(spell.is_active && spell.spell_duration > 0) {
                spell.SpellThink(player);
                if(spell.spell_start_time != -1) {
                    spell.SpellThinkDuration(player);
                }
            }
        }
    }
}