class Spells.CustomSpells.SpellSentryBuster extends Spells.CustomSpell {
    // For custom spells, this determines the icon, not the functionality
    spell_id = Spells.ESpellID.SPELL_KART_BOMB;

    is_rare_spell = false;
    cast_delay = 0.2;
    cast_sound = "Halloween.spell_fireball_cast";
    cast_response = "TLK_PLAYER_CAST_FIREBALL";

    spell_duration = 10; // 0 = instantaneous
    is_active = false;

    // spell-specific variables
    sentry_buster_model = null;
    explode_time = -1;

    function TogglePlayerCosmetics(player, enabled) {
        for (local wearable = player.FirstMoveChild(); wearable != null; wearable = wearable.NextMovePeer())
        {
            if (wearable.GetClassname() != "tf_wearable") continue;
            if(enabled) wearable.EnableDraw(); else wearable.DisableDraw();
        }
    }

    function OnCast(player) {
        local scope = player.GetScriptScope();
        scope.spells.can_cast_spell = false;

        player.SetForcedTauntCam(1);
        player.AddCustomAttribute("override footstep sound set", 6, -1);
        player.AddCustomAttribute("no_attack", 1, -1);
        player.AddCustomAttribute("max health additive bonus", (2500 - player.GetMaxHealth()), -1);
        player.SetHealth(2500);
        player.AddCond(Constants.ETFCond.TF_COND_PREVENT_DEATH);

        TogglePlayerCosmetics(player, false);
        NetProps.SetPropInt(player, "m_nRenderMode", Constants.ERenderMode.kRenderNone);
        player.SetModelScale(1.5, 1);

        // temp (for jam): no red sentry buster skin, so just tint it for now
        local rendercolor = (player.GetTeam() == 2 ? "255 64 64" : "255 255 255");

        // bonemerge a sentry buster model onto the player
        local wearable = SpawnEntityFromTable("tf_wearable", { "targetname": "model_sb", "rendermode": 1, "rendercolor": rendercolor });
        wearable.SetModelSimple("models/bots/demo/bot_sentry_buster.mdl");
        EntFireByHandle(wearable, "SetParent", "!activator", 0, player, player);
        NetProps.SetPropEntity(wearable, "m_hOwnerEntity", player);

        PrecacheScriptSound("MVM.SentryBusterIntro");
        PrecacheScriptSound("MVM.SentryBusterLoop");
        PrecacheScriptSound("MVM.SentryBusterSpin");
        EmitSoundEx({
            sound_name = "MVM.SentryBusterIntro",
            origin = player.GetOrigin(),
            sound_level = 80,
            flags = 0,
            filter_type = 5
        });
        EmitAmbientSoundOn("MVM.SentryBusterLoop", 1, 80, 100, player);

        sentry_buster_model = wearable;
    }

    function SpellThink(player) {
        // force scout movement speed
        NetProps.SetPropFloat(player, "m_flMaxspeed", 400);

        // prevent death from taking damage (still buggy because Condition Sux™)
        if(!player.InCond(Constants.ETFCond.TF_COND_PREVENT_DEATH) && explode_time == -1) {
            player.AddFlag(Constants.FPlayer.FL_GODMODE);
            explode_time = Time();
            ExplodeStart(player);
        }

        local buttons = NetProps.GetPropInt(player, "m_nButtons");
        if(explode_time == -1) {
            local ground_ent = NetProps.GetPropEntity(player, "m_hGroundEntity");
            if(ground_ent == null) return;

            if(buttons & Constants.FButtons.IN_ATTACK2) {
                player.AddFlag(Constants.FPlayer.FL_GODMODE);
                explode_time = Time();
                ExplodeStart(player);
            }
        } else if(explode_time != -2) {
            if(Time() - explode_time > 2) {
                ExplodeEnd(player);
                explode_time = -2;
            }
        }
    }

    function ExplodeStart(player) {
        player.SetAbsVelocity(Vector(0,0,0));
        player.SetMoveType(Constants.EMoveType.MOVETYPE_NONE, Constants.EMoveCollide.MOVECOLLIDE_DEFAULT);
        StopAmbientSoundOn("MVM.SentryBusterLoop", player);

        EmitSoundEx({
            sound_name = "MVM.SentryBusterSpin",
            origin = player.GetOrigin(),
            sound_level = 120,
            flags = 0,
            filter_type = 5
        });

        sentry_buster_model.Kill();

        // temp (for jam): no red sentry buster skin, so just tint it for now
        local rendercolor = (player.GetTeam() == 2 ? "255 64 64" : "255 255 255");

        sentry_buster_model = SpawnEntityFromTable("prop_dynamic", {
            "angles": player.GetAbsAngles(),
            "model": "models/bots/demo/bot_sentry_buster.mdl",
            "DefaultAnim": "sentry_buster_preExplode",
            "origin": player.GetOrigin(),
            "modelscale": 1.5,
            "rendermode": 1, "rendercolor": rendercolor
        });
        spell_start_time = -1;
    }

    function ExplodeEnd(player) {
        PrecacheScriptSound("BaseExplosionEffect.Sound");
        // actually explode
        local bomb = SpawnEntityFromTable("tf_generic_bomb", {
            "origin": player.GetOrigin(),
            "damage": 500,
            "radius": 256,
            "explode_particle": "ExplosionCore_MidAir",
            "sound": "BaseExplosionEffect.Sound"
        });
        bomb.TakeDamageEx(player, player, player.GetActiveWeapon(), Vector(0,0,0), Vector(0,0,0), 500, Constants.FDmgType.DMG_BLAST);

        player.RemoveFlag(Constants.FPlayer.FL_GODMODE);

        // blast player into the air instead of killing them
        local velo = player.GetAbsVelocity();
        player.SetAbsVelocity(Vector(velo.x, velo.y, velo.z + 600));

        player.SetMoveType(Constants.EMoveType.MOVETYPE_WALK, Constants.EMoveCollide.MOVECOLLIDE_DEFAULT);
        Cleanup(player);
    }

    function Cleanup(player) {
        local scope = player.GetScriptScope();
        scope.spells.can_cast_spell = true;

        is_active = false;
        player.SetForcedTauntCam(0);
        player.RemoveCustomAttribute("override footstep sound set");
        player.RemoveCustomAttribute("max health additive bonus");
        player.RemoveCustomAttribute("no_attack");
        NetProps.SetPropInt(player, "m_nRenderMode", Constants.ERenderMode.kRenderNormal);

        TogglePlayerCosmetics(player, true);
        player.SetModelScale(1, 0);
        player.SetHealth(player.GetMaxHealth());
        StopAmbientSoundOn("MVM.SentryBusterLoop", player);

        if(sentry_buster_model != null && sentry_buster_model.IsValid()) sentry_buster_model.Kill();
    }

    function OnEnd(player) {
        if(explode_time == -1) {
            // if a player killbinded, let them die
            printl(NetProps.GetPropInt(player, "m_lifeState"));
            if(NetProps.GetPropInt(player, "m_lifeState") != 0) {
                Cleanup(player);
                return;
            }
            // otherwise, time has elapsed: time to explode!
            player.AddFlag(Constants.FPlayer.FL_GODMODE);
            explode_time = Time();
            ExplodeStart(player);
            is_active = true;
            spell_start_time = -1;
        }
        else {
            Cleanup(player);
        }
    }

}