::MaxPlayers <- MaxClients().tointeger();

pickup_radius <- 64;
can_pickup <- true;
tier <- 0;
auto_materialize <- true;

function Init(spell_tier, auto_mat) {
	PrecacheScriptSound( "Halloween.spell_pickup" );
	PrecacheScriptSound( "Halloween.spell_pickup_rare" );
	PrecacheScriptSound( "Item.Materialize" );

    tier = spell_tier;
    auto_materialize = auto_mat;
}

function GetPlayerSpellbook(player) {
    local weapons_count = NetProps.GetPropArraySize(player, "m_hMyWeapons")
    for (local i = 0; i < weapons_count; i++) {
        local weapon = NetProps.GetPropEntityArray(player, "m_hMyWeapons", i)
        if(weapon != null && weapon.IsValid() && weapon.GetClassname() == "tf_weapon_spellbook") {
            return weapon;
        }
    }
    return null;
}

function DoPickup(player) {
    can_pickup = false;
    EntFireByHandle(self, "Disable", "", 0, null, null);

    if(auto_materialize) {
        EntFireByHandle(self, "CallScriptFunction", "ResetPickup", 10, null, null);
    }

    // make the pickup sound specific to the tier
    local pickup_sound = Spells.SpellTiers[tier].pickup_sound;
    PrecacheScriptSound(pickup_sound);
    EmitSoundEx({
        sound_name = pickup_sound,
        origin = self.GetOrigin(),
        sound_level = 80,
        flags = 0,
        filter_type = 5
    });

    // roll the spell from the correct tier's pool
    local roll = Spells.RollSpell(tier);

    // initialize the spell
    local spell = roll["spell"]();
    local is_rare = spell.is_rare_spell;
    local charges = roll["charges"];

    EntFireByHandle(self, "CallScriptFunction", "RollStart", 0, player, player);
    EntFireByHandle(self, "RunScriptCode", format("RollEnd(%d, %s, %d)", spell.spell_id, is_rare.tostring(), charges), Spells.Config.spell_roll_time, player, player);

    // if this is a custom spell, override the spell roll to an out-of-range value.
    // this will remove all ability to use the spell normally, which means we don't need to work around existing effects,
    //   but it means the entire spell casting system needs to be recreated manually.
    // a bonus of this (and a core of this system) is that the system will always use the icon of the last in-bounds spell ID, so we can pick one we want
    if(spell.is_custom_spell) {
        local index = (spell.is_rare_spell) ? 20 : -20;
        EntFireByHandle(self, "RunScriptCode", format("RollEnd(%d, %s, %d)", index, is_rare.tostring(), charges), Spells.Config.spell_roll_time + 0.05, player, player);
    }

    local scope = player.GetScriptScope();
    scope.spells.spellbook_spell <- spell;
}

function RollStart() {
    local player = activator;
    local spellbook = GetPlayerSpellbook(player);
    if(spellbook != null) {
        NetProps.SetPropInt(spellbook, "m_iSelectedSpellIndex", -2);
    }
}

function RollEnd(index, is_rare, charges = -1) {
    local player = activator;
    local spellbook = GetPlayerSpellbook(player);
    if(spellbook != null) {
        NetProps.SetPropInt(spellbook, "m_iSelectedSpellIndex", index);
        if(charges != -1) NetProps.SetPropInt(spellbook, "m_iSpellCharges", charges);
    }

    local concept = (is_rare) ? "TLK_PLAYER_SPELL_PICKUP_RARE" : "TLK_PLAYER_SPELL_PICKUP_COMMON";
    EntFireByHandle(player, "SpeakResponseConcept", concept, 0, null, null);
}

function Distance(a, b) {
    return (b-a).Length();
}

function PickupRangeLogic() {
    for (local i = 1; i <= MaxPlayers; i++)
    {
        local player = PlayerInstanceFromIndex(i);
        if (player == null) continue;

        local lifestate = NetProps.GetPropInt(player, "m_lifeState");
        if(lifestate != 0) continue;

        if(Distance(self.GetOrigin(), player.GetOrigin()) <= pickup_radius) {
            if(!CanPickUpSpell(player)) { continue; }

            DoPickup(player);
        }
    }
}

function ResetPickup() {
    EmitSoundEx({
        sound_name = "Item.Materialize",
        origin = self.GetOrigin(),
        sound_level = 80,
        flags = 0,
        filter_type = 5
    });

    EntFireByHandle(self, "Enable", "", 0, null, null);
    can_pickup = true;
}

function CanPickUpSpell(player) {
    local spellbook = GetPlayerSpellbook(player);

    if(spellbook != null) {
        // always pick up rare spells
        if(tier >= 1) return true;

        // don't pick up a spell if we already have one or are rolling one
        if(NetProps.GetPropInt(spellbook, "m_iSpellCharges") > 0 || NetProps.GetPropInt(spellbook, "m_iSelectedSpellIndex") == -2) return false;

        return true;
    }

    // this string is unlocalized in regular TF2, so there's no need to supply a localization string until this gets updated
    ClientPrint(player, Constants.EHudNotify.HUD_PRINTCENTER, "Equip a SpellBook in your ActionSlot to pick this up.");
    return false;
}

function Think() {
    if(can_pickup) { PickupRangeLogic(); }
    return 0;
}